'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var define = require('./define-18cb4958.js');
var graphql = require('graphql');

function buildGraphQLSchema(schema) {
    const typeMap = new Map();
    return new graphql.GraphQLSchema({
        query: toGraphQLOutputType(schema.query, typeMap),
        mutation: schema.mutation &&
            toGraphQLOutputType(schema.mutation, typeMap),
        subscription: schema.subscription &&
            toGraphQLSubscriptionObject(schema.subscription, typeMap),
        types: schema.types &&
            schema.types.map((type) => toGraphQLOutputType(type, typeMap)),
        directives: schema.directives,
    });
}
function toGraphQLArgs(args, typeMap) {
    const graphqlArgs = {};
    Object.keys(args).forEach((k) => {
        const arg = args[k];
        graphqlArgs[k] = {
            type: toGraphQLInputType(arg.type, typeMap),
            description: arg.description,
            defaultValue: arg.default,
        };
    });
    return graphqlArgs;
}
function toGraphQLSubscriptionObject(subscriptionObj, typeMap) {
    return new graphql.GraphQLObjectType({
        name: subscriptionObj.name,
        fields: () => {
            const gqlFieldConfig = {};
            subscriptionObj.fields().forEach((field) => {
                gqlFieldConfig[field.name] = {
                    type: toGraphQLOutputType(field.type, typeMap),
                    description: field.description,
                    subscribe: field.subscribe,
                    resolve: field.resolve,
                    args: toGraphQLArgs(field.args, typeMap),
                    deprecationReason: field.deprecationReason,
                };
            });
            return gqlFieldConfig;
        },
    });
}
function toGraphQLInputType(t, typeMap) {
    const found = typeMap.get(t);
    if (found) {
        return typeMap.get(t);
    }
    switch (t.kind) {
        case 'Scalar':
        case 'Enum':
            return toGraphQLOutputType(t, typeMap);
        case 'NonNullInput':
            return new graphql.GraphQLNonNull(toGraphQLInputType(t.ofType, typeMap));
        case 'ListInput':
            return new graphql.GraphQLList(toGraphQLInputType(t.ofType, typeMap));
        case 'InputObject':
            const fields = t.fieldsFn();
            function graphqlFields() {
                const gqlFieldConfig = {};
                Object.keys(fields).forEach((k) => {
                    const field = fields[k];
                    gqlFieldConfig[k] = {
                        type: toGraphQLInputType(field.type, typeMap),
                        description: field.description,
                        defaultValue: field.defaultValue,
                        deprecationReason: field.deprecationReason,
                    };
                });
                return gqlFieldConfig;
            }
            const obj = new graphql.GraphQLInputObjectType({
                name: t.name,
                description: t.description,
                fields: graphqlFields,
            });
            typeMap.set(t, obj);
            return obj;
    }
}
function toGraphQLOutputType(t, typeMap) {
    const found = typeMap.get(t);
    if (found) {
        return found;
    }
    switch (t.kind) {
        case 'Scalar':
            let scalar;
            if ('builtInType' in t) {
                scalar = t.builtInType;
            }
            else {
                scalar = new graphql.GraphQLScalarType({
                    ...t.graphqlTypeConfig,
                });
            }
            typeMap.set(t, scalar);
            return scalar;
        case 'Enum':
            const enumT = new graphql.GraphQLEnumType({
                name: t.name,
                description: t.description,
                values: t.values.reduce((acc, val) => {
                    acc[val.name] = {
                        value: val.value,
                        deprecationReason: val.deprecationReason,
                        description: val.description,
                    };
                    return acc;
                }, {}),
            });
            typeMap.set(t, enumT);
            return enumT;
        case 'NonNull':
            return new graphql.GraphQLNonNull(toGraphQLOutputType(t.ofType, typeMap));
        case 'List':
            return new graphql.GraphQLList(toGraphQLOutputType(t.ofType, typeMap));
        case 'ObjectType':
            const obj = new graphql.GraphQLObjectType({
                name: t.name,
                description: t.description,
                interfaces: (typeof t.interfaces === 'function'
                    ? t.interfaces()
                    : t.interfaces).map((intf) => toGraphQLOutputType(intf, typeMap)),
                isTypeOf: t.isTypeOf,
                fields: () => {
                    const fields = t.fieldsFn();
                    const gqlFieldConfig = {};
                    fields.forEach((field) => {
                        gqlFieldConfig[field.name] = {
                            type: toGraphQLOutputType(field.type, typeMap),
                            description: field.description,
                            resolve: field.resolve,
                            args: toGraphQLArgs(field.args, typeMap),
                            deprecationReason: field.deprecationReason,
                            extensions: field.extensions,
                        };
                    });
                    return gqlFieldConfig;
                },
                extensions: t.extensions,
            });
            typeMap.set(t, obj);
            return obj;
        case 'Union':
            const types = typeof t.types === 'function' ? t.types() : t.types;
            const union = new graphql.GraphQLUnionType({
                name: t.name,
                description: t.description,
                types: types.map((t) => toGraphQLOutputType(t, typeMap)),
                resolveType: t.resolveType
                    ? async (src, ctx, info, abstractType) => {
                        var _a;
                        let ourType;
                        for (const [t, graphqlType] of typeMap.entries()) {
                            if (graphqlType === abstractType) {
                                ourType = t;
                                break;
                            }
                        }
                        return (_a = t.resolveType) === null || _a === void 0 ? void 0 : _a.call(t, src, ctx, info, ourType);
                    }
                    : undefined,
            });
            typeMap.set(t, union);
            return union;
        case 'Interface':
            const intf = new graphql.GraphQLInterfaceType({
                name: t.name,
                description: t.description,
                fields: () => {
                    const fields = t.fieldsFn();
                    const result = {};
                    fields.forEach((field) => {
                        result[field.name] = {
                            type: toGraphQLOutputType(field.type, typeMap),
                            description: field.description,
                            deprecationReason: field.deprecationReason,
                            args: field.args && toGraphQLArgs(field.args, typeMap),
                        };
                    });
                    return result;
                },
                interfaces: (typeof t.interfaces === 'function'
                    ? t.interfaces()
                    : t.interfaces).map((intf) => toGraphQLOutputType(intf, typeMap)),
                resolveType: t.resolveType
                    ? async (src, ctx, info, abstractType) => {
                        var _a;
                        let ourType;
                        for (const [t, graphqlType] of typeMap.entries()) {
                            if (graphqlType === abstractType) {
                                ourType = t;
                                break;
                            }
                        }
                        return (_a = t.resolveType) === null || _a === void 0 ? void 0 : _a.call(t, src, ctx, info, ourType);
                    }
                    : undefined,
            });
            typeMap.set(t, intf);
            return intf;
    }
}

Object.defineProperty(exports, 'Gql', {
    enumerable: true,
    get: function () {
        return define.Gql;
    }
});
exports.buildGraphQLSchema = buildGraphQLSchema;
exports.toGraphQLInputType = toGraphQLInputType;
exports.toGraphQLOutputType = toGraphQLOutputType;
